import System from 'corpjs-system'
import winston = require('winston')

export function Logger(): System.Component {

  let logger: winston.LoggerInstance

  return {
    async start({config}: { config: Logger.Config }, _, stop): Promise<winston.LoggerInstance> {
      return logger = new winston.Logger(prepareConfig(config || {})).on('error', stop)
    },

    async stop() {
      if (logger)
        logger.close()
    }
  }

}

export namespace Logger {

  export interface Config extends winston.LoggerOptions {
    transportFactories?: (() => winston.TransportInstance)[]
  }

  export const transports = winston.transports

}

function prepareConfig(config) {
  const transports = (config.transportFactories || [])
    .map(factory => factory())
  return { ...config, transports }
}

export default Logger
export const transports = winston.transports