import 'mocha'
import * as assert from 'assert'
import System from 'corpjs-system'
import Logger, { transports } from '../src'
import freeze = require('deep-freeze')

describe('corpjs-logger', () => {

  let system

  beforeEach(async () => {
    if (system) await system.stop()
  })

  afterEach(async () => {
    if (system) await system.stop()
  })

  it('should has transports', () => {
    assert.ok(transports)
    assert.ok(Logger.transports)
  })

  it('should start', async () => {
    system = createSystem({})
    const {logger} = await system.start()
    assert.ok(logger)
    assert.ok(logger.log)
  })

  it('should create and use File transporter', done => {
    system = createSystem({
      transportFactories: [
        () => new transports.File({ filename: 'test/test.log' })
      ]
    })
    system.start().then(({logger}) => {
      assert.ok(logger)
      assert.ok(logger.log)
      logger.once('logging', () => done())
      logger.info(`test`)
    }).catch(done)
  })

})

function createSystem(loggerConfig: Logger.Config): System {
  return new System({ exitOnError: false })
    .add('config', Config(loggerConfig))
    .add('logger', Logger()).dependsOn({ component: 'config', source: 'logger', as: 'config' })
}

function Config(loggerConfig) {
  return {
    async start() {
      return freeze({ logger: loggerConfig })
    }
  }
}